//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "sockets.h"
#include "types.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "searchlist.h"
#include <time.h>
#include <afxmt.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

void CClientSocket::OnAccept(int nErrorCode){
	MessageBox(0,"Accept","Test",64);
}
CServerSocket::CServerSocket(CServerConnect* in_serverconnect){
	serverconnect = in_serverconnect;
	headercomplete = false;
	connectionstate = false;
	sizereceived = 0;
	sizetoget = 0;
	cur_server = 0;
}
void CServerSocket::OnConnect(int nErrorCode){
	switch (nErrorCode){
		case 0:
			SetConnectionState(CS_WAITFORLOGIN);
			break;
		case WSAEADDRNOTAVAIL:
		case WSAECONNREFUSED:
		case WSAENETUNREACH:
		case WSAETIMEDOUT:
			theApp.emuledlg->AddLogLine(false,"%s (%s:%i) appears to be dead",cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort()); 		
			SetConnectionState(CS_SERVERDEAD);
			return;
		default:	
			SetConnectionState(CS_FATALERROR);
			return;
	}
	
	 CAsyncSocket::OnConnect(nErrorCode);	 
}
void CServerSocket::OnReceive(int nErrorCode){
	if  (!sizetoget || !headercomplete){
		if (!sizetoget){
			rbuffer = new char[5];
			headercomplete = false;
			sizereceived = this->Receive(rbuffer,5);
		}
		else
			sizereceived += this->Receive(rbuffer+sizereceived,sizetoget-sizereceived);

		if (sizereceived < 5){
			theApp.emuledlg->AddLogLine(false,"Error in Server:OnReceive! size < 5");
			sizetoget = 5;
			return;
		}
		else{
			Header_Struct* header = (Header_Struct*) rbuffer;
			if (header->eDonkeyID != OP_EDONKEYHEADER){
				theApp.emuledlg->AddLogLine(false,"Debug: Error in Server:OnReceive! header invalid");
				// errorcorrection: try to find next header somewhere		
				for (int i = 0;i != 5; i++){
					if (rbuffer[i] == OP_EDONKEYHEADER){
						memmove(rbuffer,rbuffer+i,5-i);
						sizetoget = 5;
						sizereceived = 5-i;
						return;
					}
				}
				delete rbuffer;
				sizereceived = 0;
				return;
			}
			else {
				headercomplete = true;
				sizetoget = header->packetlength;
				sizereceived = 0;
				delete rbuffer;
				return;
			}
		}
	}
	else {
		if (!rbuffer || !sizereceived)
			rbuffer = new char[sizetoget];
		sizereceived += this->Receive(rbuffer+sizereceived,sizetoget-sizereceived);
		if (sizetoget == sizereceived){
			ProcessPacket(rbuffer+1, sizetoget-1, rbuffer[0]);
			delete[] rbuffer;
			sizetoget = 0;
			sizereceived = 0;
		}
	}
	CAsyncSocket::OnReceive(nErrorCode);
}

bool CServerSocket::ProcessPacket(char* packet, int32 size, int8 opcode){
	if (connectionstate != CS_CONNECTED)
		SetConnectionState(CS_CONNECTED);
	switch(opcode){
		case OP_SERVERMESSAGE:{
			ServerMessage_Struct* sm = (ServerMessage_Struct*) packet;
			if (sm->size != size-2)
				theApp.emuledlg->AddLogLine(false,"Corrupt or invalid welcomemessage from server received");
			char* buffer = new char[size-1];
			memcpy(buffer,sm->message,size-2);
			buffer[size-2] = 0;
			theApp.emuledlg->AddServerMessageLine(buffer);
			delete []buffer;
			break;
		}
		case OP_IDCHANGE:{
			if (size != sizeof(LoginAnswer_Struct)){
				theApp.emuledlg->AddLogLine(false,"Corrupt or invalid loginanswer from server received");
				SetConnectionState(CS_ERROR);
				break;
			}
			LoginAnswer_Struct* la = (LoginAnswer_Struct*) packet;
			clientid = la->clientid;
			theApp.emuledlg->AddLogLine(false,"New clientid is %i",clientid);
			
			break;
		}
		case OP_SEARCHRESULT:{
			theApp.searchlist->ProcessSearchanswer(packet,size);
			theApp.emuledlg->searchwnd.LocalSearchEnd();
			break;
		}
		default:
			return false;
	}
	return true;
}

void CServerSocket::ConnectToServer(CServer* server){
	cur_server = server;
	theApp.emuledlg->AddLogLine(false,"Connecting to %s (%s:%i)...",cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort());
	SetConnectionState(CS_CONNECTING);
	if (!this->Connect(server->GetFullIP(),server->GetPort())){
		int error = this->GetLastError();
		if ( error != WSAEWOULDBLOCK){
			theApp.emuledlg->AddLogLine(false,"Error while connecting to %s (%s:%i)",cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort()); 
			SetConnectionState(CS_FATALERROR);
			return;
		}
	}
	SetConnectionState(CS_CONNECTING);
}

void CServerSocket::OnClose(int nErrorCode){
	if (connectionstate == CS_CONNECTING){	
		theApp.emuledlg->AddLogLine(false,"%s (%s:%i) appears to be full",cur_server->GetListName(),cur_server->GetFullIP(),cur_server->GetPort()); 	
		SetConnectionState(CS_SERVERFULL);
	}
	else if (connectionstate == CS_CONNECTED){
		SetConnectionState(CS_DISCONNECTED);		
	}
	else{
		SetConnectionState(CS_NOTCONNECTED);
	}
	CAsyncSocket::OnClose(nErrorCode);
}

void CServerSocket::SetConnectionState(sint8 newstate){
	connectionstate = newstate;
	if (newstate < 1){
		if (this->serverconnect)
			serverconnect->ConnectionFailed(this);
	}
	else if (newstate == CS_CONNECTED || newstate == CS_WAITFORLOGIN){
		if (serverconnect)
			serverconnect->ConnectionEstablished(this);
	}
}


void CUDPSocket::OnReceive(int nErrorCode){
	char* buffer = new char[5000];
	int16 length = Receive(buffer,5000);
	if ((uint8)buffer[0] == OP_EDONKEYHEADER)
		ProcessPacket(buffer+2,length-2,buffer[1]);
	delete[] buffer;
}

bool CUDPSocket::ProcessPacket(char* packet, int16 size, int8 opcode){
	switch(opcode){
		case OP_GLOBSEARCHRES:{
			theApp.emuledlg->searchwnd.AddUDPResult(theApp.searchlist->ProcessUDPSearchanswer(packet,size));
			break;
		}
		default:
			return false;
	}
	return true;
}



void CServerConnect::ConnectToAnyServer(){
	StopConnectionTry();
	Disconnect();
	connecting = true;
	max_simcons = (used_list->GetServerCount() < max_simcons) ? used_list->GetServerCount():max_simcons;
	sockets = new CServerSocket*[max_simcons];
	for (int i = 0;i != max_simcons; i++){
		sockets[i] = new CServerSocket(this);
		sockets[i]->Create(0,SOCK_STREAM,FD_READ|FD_WRITE|FD_CLOSE|FD_CONNECT,NULL);
	}
	for (int i = 0;i != max_simcons; i++){
		CServer*  next_server;
		next_server = used_list->GetNextServer();
		if (next_server)
			sockets[i]->ConnectToServer(next_server);
		else {
			theApp.emuledlg->AddLogLine(true,"No valid servers to connect in serverlist found");
			StopConnectionTry();
			return;
		}
	}
}
void CServerConnect::StopConnectionTry(){
	connecting = false;
	if (!sockets)
		return;
	for (int i = 0;i != max_simcons; i++){
		if (sockets[i] != connectedsocket){ 
			sockets[i]->Close();
			delete sockets[i];
		}
	}
	delete []sockets;
	sockets = 0;	
}
void CServerConnect::ConnectionEstablished(CServerSocket* sender){
	if (sender->GetConnectionState() == CS_WAITFORLOGIN){
		theApp.emuledlg->AddLogLine(false,"Connected to %s (%s:%i), sending loginrequest",sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort());
		Packet* packet = new Packet(OP_LOGINREQUEST,sizeof(LoginP1_Struct)+sizeof(LoginP2_Struct)+strlen(app_prefs->GetUserNick()));
		LoginP1_Struct* login1 = (LoginP1_Struct*)packet->pBuffer;
		LoginP2_Struct* login2 = (LoginP2_Struct*)&packet->pBuffer[sizeof(LoginP1_Struct)+strlen(app_prefs->GetUserNick())];

		login1->unkown_01 = 0x3;
		login1->unkown_02 = 0x01000102;
		srand((uint32)time(0));		
		for (int i = 0;i != 4; i++){ // make userhash random, this will cause problems with friend functions, but they aren't in yet anyway
			uint32	random = rand();
			memcpy(login1->userhash+(i*4),&random,4);
		}
		login1->username_len = strlen(app_prefs->GetUserNick());
		memcpy(login1->username,app_prefs->GetUserNick(),strlen(app_prefs->GetUserNick()));
		login1->clientport_1 = app_prefs->GetPort();
		login2->clientport_2 = app_prefs->GetPort();
		login2->unknown_03 = 0x11000103;
		login2->unknown_04 = 0x0f000103;
		login2->version = 0x3c;
		login2->unknown_05 = 0;
		this->SendPacket(packet,true,sender);
	}
	else if (sender->GetConnectionState() == CS_CONNECTED){	
		connected = true;
				theApp.emuledlg->AddLogLine(true,"Connection established on %s",sender->cur_server->GetListName());
		theApp.emuledlg->ShowConnectionState(true);
		connectedsocket = sender;
		StopConnectionTry();
		theApp.sharedfiles->SendListToServer();
	}

}
bool CServerConnect::SendPacket(Packet* packet,bool delpacket, CServerSocket* to){
	if (!to){
		if (connected){
			connectedsocket->Send(packet->GetHeader(),6);
			connectedsocket->Send(packet->pBuffer,packet->size);
			if (delpacket)
				delete packet;
		}
		else
			return false;
	}
	else{
		to->Send(packet->GetHeader(),6);
		to->Send(packet->pBuffer,packet->size);
		if (delpacket)
			delete packet;
	}
	return true;
}

bool CServerConnect::SendUDPPacket(Packet* packet,char* host, uint16 port,bool delpacket){
		if (connected){
			udpsocket->Connect(host,port);
			char* buffer = new char[packet->size+2];
			memcpy(buffer,packet->GetUDPHeader(),2);
			memcpy(buffer+2,packet->pBuffer,packet->size);
			udpsocket->Send(buffer,packet->size+2);
			delete[] buffer;
			if (delpacket)
				delete packet;
		}
		else
			return false;
	return true;
}

void CServerConnect::ConnectionFailed(CServerSocket* sender){
	switch (sender->GetConnectionState()){
				case CS_FATALERROR:{
					theApp.emuledlg->AddLogLine(true,"Fatal Error while trying to connect. Internet connection might be down");
					StopConnectionTry();
					break;
				}
				case CS_DISCONNECTED:{
					theApp.emuledlg->AddLogLine(true,"Lost connection to %s (%s:%i)",sender->cur_server->GetListName(),sender->cur_server->GetFullIP(),sender->cur_server->GetPort());
					connected = false;
					theApp.emuledlg->ShowConnectionState(false);
					sender->Close();
					delete sender;
					connectedsocket = 0;
					theApp.emuledlg->searchwnd.OnBnClickedCancels();
					if (app_prefs->Reconnect() && !connecting){
						ConnectToAnyServer();		
					}
					break;
									 }
				case CS_SERVERDEAD:{
					//TODO mark server as dead
								   }
				case CS_ERROR:
				case CS_SERVERFULL:
				case CS_NOTCONNECTED:{
					if (!connecting)
						break;
					sender->Close();
					sender->Create(0,SOCK_STREAM,FD_READ|FD_WRITE|FD_CLOSE|FD_CONNECT,NULL);			
					CServer* next_server = used_list->GetNextServer();
					if (next_server)
						sender->ConnectToServer(next_server);
					else {
						theApp.emuledlg->AddLogLine(true,"No valid servers to connect in serverlist found");
						StopConnectionTry();
						return;
					}
				}
	}
}
bool CServerConnect::Disconnect(){
	if (connected && connectedsocket){
		connectedsocket->Close();
		delete connectedsocket;
		connected = false;
		theApp.emuledlg->ShowConnectionState(false);
		return true;
	}
	else
		return false;
}
CServerConnect::CServerConnect(CServerList* in_serverlist, CPreferences* in_prefs){
	connectedsocket = 0;
	sockets = 0;
	used_list = in_serverlist;
	max_simcons = 10;
	connecting = false;
	connected = false;
	app_prefs = in_prefs;

	udpsocket = new CUDPSocket(this); // initalize socket for udp packets
	udpsocket->Create(0,SOCK_DGRAM);
}
CServerConnect::~CServerConnect(){
	StopConnectionTry();
	safe_delete(connectedsocket);
	udpsocket->Close();
	delete udpsocket;
}

CServer* CServerConnect::GetCurrentServer(){
	if (IsConnected() && connectedsocket)
		return connectedsocket->cur_server;
	return 0;
}